@@ -416,7 +416,7 @@ XKeycodeToKeysym(
  *----------------------------------------------------------------------
  */
 
-char *
+const char *
 TkpGetString(
     TkWindow *winPtr,		/* Window where event occurred: Needed to get
 				 * input context. */
@@ -458,7 +458,7 @@ XGetModifierMapping(
      * don't generate them either. So there is no modifier map.
      */
 
-    modmap = (XModifierKeymap *) ckalloc(sizeof(XModifierKeymap));
+    modmap = ckalloc(sizeof(XModifierKeymap));
     modmap->max_keypermod = 0;
     modmap->modifiermap = NULL;
     return modmap;
@@ -485,9 +485,9 @@ XFreeModifiermap(
     XModifierKeymap *modmap)
 {
     if (modmap->modifiermap != NULL) {
-	ckfree((char *) modmap->modifiermap);
+	ckfree(modmap->modifiermap);
     }
-    ckfree((char *) modmap);
+    ckfree(modmap);
     return Success;
 }
 
@@ -734,6 +734,7 @@ TkpGetKeySym(
      */
 
     if (eventPtr->xany.send_event == -1) {
+
 	int modifier = eventPtr->xkey.keycode & NSDeviceIndependentModifierFlagsMask;
 
 	if (modifier == NSCommandKeyMask) {
@@ -898,7 +899,7 @@ TkpInitKeymapInfo(
      */
 
     if (dispPtr->modKeyCodes != NULL) {
-	ckfree((char *) dispPtr->modKeyCodes);
+	ckfree(dispPtr->modKeyCodes);
     }
     dispPtr->numModKeyCodes = 0;
     dispPtr->modKeyCodes = NULL;
