@@ -9,10 +9,11 @@
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 
-package require Tcl 8.4
-package require tcltest 2.1
 package require Tk ;# This is the Tk test suite; fail early if no Tk!
-tcltest::configure -testdir [file join [pwd] [file dirname [info script]]]
+package require tcltest 2.2
+tcltest::configure {*}$argv
+tcltest::configure -testdir [file normalize [file dirname [info script]]]
+tcltest::configure -loadfile \
+	[file join [tcltest::testsDirectory] constraints.tcl]
 tcltest::configure -singleproc 1
-eval tcltest::configure $argv
 tcltest::runAllTests
