@@ -28,8 +28,8 @@ typedef struct {
  * the official cursor font:
  */
 
-static struct CursorName {
-    CONST char *name;
+static const struct CursorName {
+    const char *name;
     unsigned int shape;
 } cursorNames[] = {
     {"X_cursor",		XC_X_cursor},
@@ -159,9 +159,9 @@ static struct CursorName {
 
 #endif /* DEFINE_MYARROW_CURSOR */
 
-static struct TkCursorName {
-    char *name;
-    char *data;
+static const struct TkCursorName {
+    const char *name;
+    const char *data;
     char *mask;
 } tkCursorNames[] = {
     {"none",	CURSOR_NONE_DATA,	NULL},
@@ -180,8 +180,8 @@ static struct TkCursorName {
 #endif
 
 static Cursor		CreateCursorFromTableOrFile(Tcl_Interp *interp,
-			    Tk_Window tkwin, int argc, CONST char **argv,
-			    struct TkCursorName *tkCursorPtr);
+			    Tk_Window tkwin, int argc, const char **argv,
+			    const struct TkCursorName *tkCursorPtr);
 
 /*
  *----------------------------------------------------------------------
@@ -211,10 +211,10 @@ TkGetCursorByName(
     TkUnixCursor *cursorPtr = NULL;
     Cursor cursor = None;
     int argc;
-    CONST char **argv = NULL;
+    const char **argv = NULL;
     Display *display = Tk_Display(tkwin);
     int inTkTable = 0;
-    struct TkCursorName* tkCursorPtr = NULL;
+    const struct TkCursorName *tkCursorPtr = NULL;
 
     if (Tcl_SplitList(interp, string, &argc, &argv) != TCL_OK) {
 	return NULL;
@@ -245,7 +245,7 @@ TkGetCursorByName(
     if ((argv[0][0] != '@') && !inTkTable) {
 	XColor fg, bg;
 	unsigned int maskIndex;
-	register struct CursorName *namePtr;
+	register const struct CursorName *namePtr;
 	TkDisplay *dispPtr;
 
 	/*
@@ -275,8 +275,9 @@ TkGetCursorByName(
 	    bg.red = bg.green = bg.blue = 65535;
 	} else {
 	    if (TkParseColor(display, Tk_Colormap(tkwin), argv[1], &fg) == 0) {
-		Tcl_AppendResult(interp, "invalid color name \"", argv[1],
-			"\"", NULL);
+		Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+			"invalid color name \"%s\"", argv[1]));
+		Tcl_SetErrorCode(interp, "TK", "CURSOR", "COLOR", NULL);
 		goto cleanup;
 	    }
 	    if (argc == 2) {
@@ -284,8 +285,9 @@ TkGetCursorByName(
 		maskIndex = namePtr->shape;
 	    } else if (TkParseColor(display, Tk_Colormap(tkwin), argv[2],
 		    &bg) == 0) {
-		Tcl_AppendResult(interp, "invalid color name \"", argv[2],
-			"\"", NULL);
+		Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+			"invalid color name \"%s\"", argv[2]));
+		Tcl_SetErrorCode(interp, "TK", "CURSOR", "COLOR", NULL);
 		goto cleanup;
 	    }
 	}
@@ -293,7 +295,9 @@ TkGetCursorByName(
 	if (dispPtr->cursorFont == None) {
 	    dispPtr->cursorFont = XLoadFont(display, CURSORFONT);
 	    if (dispPtr->cursorFont == None) {
-		Tcl_SetResult(interp, "couldn't load cursor font", TCL_STATIC);
+		Tcl_SetObjResult(interp, Tcl_NewStringObj(
+			"couldn't load cursor font", -1));
+		Tcl_SetErrorCode(interp, "TK", "CURSOR", "FONT", NULL);
 		goto cleanup;
 	    }
 	}
@@ -306,8 +310,10 @@ TkGetCursorByName(
 	 */
 
 	if (!inTkTable && Tcl_IsSafe(interp)) {
-	    Tcl_AppendResult(interp, "can't get cursor from a file in",
-		    " a safe interpreter", NULL);
+	    Tcl_SetObjResult(interp, Tcl_NewStringObj(
+		    "can't get cursor from a file in a safe interpreter",
+		    -1));
+	    Tcl_SetErrorCode(interp, "TK", "SAFE", "CURSOR_FILE", NULL);
 	    cursorPtr = NULL;
 	    goto cleanup;
 	}
@@ -332,25 +338,26 @@ TkGetCursorByName(
     }
 
     if (cursor != None) {
-	cursorPtr = (TkUnixCursor *) ckalloc(sizeof(TkUnixCursor));
+	cursorPtr = ckalloc(sizeof(TkUnixCursor));
 	cursorPtr->info.cursor = (Tk_Cursor) cursor;
 	cursorPtr->display = display;
     }
 
   cleanup:
     if (argv != NULL) {
-	ckfree((char *) argv);
+	ckfree(argv);
     }
     return (TkCursor *) cursorPtr;
 
   badString:
     if (argv) {
-	ckfree((char *) argv);
+	ckfree(argv);
     }
-    Tcl_AppendResult(interp, "bad cursor spec \"", string, "\"", NULL);
+    Tcl_SetObjResult(interp, Tcl_ObjPrintf("bad cursor spec \"%s\"", string));
+    Tcl_SetErrorCode(interp, "TK", "VALUE", "CURSOR", NULL);
     return NULL;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -375,8 +382,8 @@ CreateCursorFromTableOrFile(
     Tcl_Interp *interp,		/* Interpreter to use for error reporting. */
     Tk_Window tkwin,		/* Window in which cursor will be used. */
     int argc,
-    CONST char **argv,		/* Cursor spec parsed into elements. */
-    struct TkCursorName *tkCursorPtr)
+    const char **argv,		/* Cursor spec parsed into elements. */
+    const struct TkCursorName *tkCursorPtr)
 				/* Non-NULL when cursor is defined in Tk
 				 * table. */
 {
@@ -386,8 +393,8 @@ CreateCursorFromTableOrFile(
     int xHot = -1, yHot = -1;
     int dummy1, dummy2;
     XColor fg, bg;
-    CONST char *fgColor;
-    CONST char *bgColor;
+    const char *fgColor;
+    const char *bgColor;
     int inTkTable = (tkCursorPtr != NULL);
 
     Display *display = Tk_Display(tkwin);
@@ -419,8 +426,9 @@ CreateCursorFromTableOrFile(
 	data = TkGetBitmapData(NULL, tkCursorPtr->data, NULL,
 		&width, &height, &xHot, &yHot);
 	if (data == NULL) {
-	    Tcl_AppendResult(interp, "error reading bitmap data for \"",
-		    argv[0], "\"", NULL);
+	    Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+		    "error reading bitmap data for \"%s\"", argv[0]));
+	    Tcl_SetErrorCode(interp, "TK", "CURSOR", "BITMAP_DATA", NULL);
 	    goto cleanup;
 	}
 
@@ -428,22 +436,24 @@ CreateCursorFromTableOrFile(
 	ckfree(data);
     } else {
 	if (TkReadBitmapFile(display, drawable, &argv[0][1],
-		(unsigned int *) &width, (unsigned int *) &height,
+		(unsigned *) &width, (unsigned *) &height,
 		&source, &xHot, &yHot) != BitmapSuccess) {
-	    Tcl_AppendResult(interp, "cleanup reading bitmap file \"",
-		    &argv[0][1], "\"", NULL);
+	    Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+		    "cleanup reading bitmap file \"%s\"", &argv[0][1]));
+	    Tcl_SetErrorCode(interp, "TK", "CURSOR", "BITMAP_FILE", NULL);
 	    goto cleanup;
 	}
     }
 
     if ((xHot < 0) || (yHot < 0) || (xHot >= width) || (yHot >= height)) {
 	if (inTkTable) {
-	    Tcl_AppendResult(interp, "bad hot spot in bitmap data for \"",
-		    argv[0], "\"", NULL);
+	    Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+		    "bad hot spot in bitmap data for \"%s\"", argv[0]));
 	} else {
-	    Tcl_AppendResult(interp, "bad hot spot in bitmap file \"",
-		    &argv[0][1], "\"", NULL);
+	    Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+		    "bad hot spot in bitmap file \"%s\"", &argv[0][1]));
 	}
+	Tcl_SetErrorCode(interp, "TK", "CURSOR", "HOTSPOT", NULL);
 	goto cleanup;
     }
 
@@ -457,8 +467,9 @@ CreateCursorFromTableOrFile(
     } else if (argc == 2) {
 	fgColor = argv[1];
 	if (TkParseColor(display, Tk_Colormap(tkwin), fgColor, &fg) == 0) {
-	    Tcl_AppendResult(interp, "invalid color name \"",
-		    fgColor, "\"", NULL);
+	    Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+		    "invalid color name \"%s\"", fgColor));
+	    Tcl_SetErrorCode(interp, "TK", "CURSOR", "COLOR", NULL);
 	    goto cleanup;
 	}
 	if (inTkTable) {
@@ -476,13 +487,15 @@ CreateCursorFromTableOrFile(
 	    bgColor = argv[3];
 	}
 	if (TkParseColor(display, Tk_Colormap(tkwin), fgColor, &fg) == 0) {
-	    Tcl_AppendResult(interp, "invalid color name \"",
-		    fgColor, "\"", NULL);
+	    Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+		    "invalid color name \"%s\"", fgColor));
+	    Tcl_SetErrorCode(interp, "TK", "CURSOR", "COLOR", NULL);
 	    goto cleanup;
 	}
 	if (TkParseColor(display, Tk_Colormap(tkwin), bgColor, &bg) == 0) {
-	    Tcl_AppendResult(interp, "invalid color name \"",
-		    bgColor, "\"", NULL);
+	    Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+		    "invalid color name \"%s\"", bgColor));
+	    Tcl_SetErrorCode(interp, "TK", "CURSOR", "COLOR", NULL);
 	    goto cleanup;
 	}
     }
@@ -511,8 +524,9 @@ CreateCursorFromTableOrFile(
 	data = TkGetBitmapData(NULL, tkCursorPtr->mask, NULL,
 		&maskWidth, &maskHeight, &dummy1, &dummy2);
 	if (data == NULL) {
-	    Tcl_AppendResult(interp, "error reading bitmap mask data for \"",
-		    argv[0], "\"", NULL);
+	    Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+		    "error reading bitmap mask data for \"%s\"", argv[0]));
+	    Tcl_SetErrorCode(interp, "TK", "CURSOR", "MASK_DATA", NULL);
 	    goto cleanup;
 	}
 
@@ -524,15 +538,17 @@ CreateCursorFromTableOrFile(
 	if (TkReadBitmapFile(display, drawable, argv[1],
 		(unsigned int *) &maskWidth, (unsigned int *) &maskHeight,
 		&mask, &dummy1, &dummy2) != BitmapSuccess) {
-	    Tcl_AppendResult(interp, "cleanup reading bitmap file \"",
-		    argv[1], "\"", NULL);
+	    Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+		    "cleanup reading bitmap file \"%s\"", argv[1]));
+	    Tcl_SetErrorCode(interp, "TK", "CURSOR", "MASK_FILE", NULL);
 	    goto cleanup;
 	}
     }
 
     if ((maskWidth != width) || (maskHeight != height)) {
-	Tcl_SetResult(interp, "source and mask bitmaps have different sizes",
-		TCL_STATIC);
+	Tcl_SetObjResult(interp, Tcl_NewStringObj(
+		"source and mask bitmaps have different sizes", -1));
+	Tcl_SetErrorCode(interp, "TK", "CURSOR", "SIZE_MATCH", NULL);
 	goto cleanup;
     }
 
@@ -568,8 +584,8 @@ CreateCursorFromTableOrFile(
 TkCursor *
 TkCreateCursorFromData(
     Tk_Window tkwin,		/* Window in which cursor will be used. */
-    CONST char *source,		/* Bitmap data for cursor shape. */
-    CONST char *mask,		/* Bitmap data for cursor mask. */
+    const char *source,		/* Bitmap data for cursor shape. */
+    const char *mask,		/* Bitmap data for cursor mask. */
     int width, int height,	/* Dimensions of cursor. */
     int xHot, int yHot,		/* Location of hot-spot in cursor. */
     XColor fgColor,		/* Foreground color for cursor. */
@@ -592,7 +608,7 @@ TkCreateCursorFromData(
     Tk_FreePixmap(display, maskPixmap);
 
     if (cursor != None) {
-	cursorPtr = (TkUnixCursor *) ckalloc(sizeof(TkUnixCursor));
+	cursorPtr = ckalloc(sizeof(TkUnixCursor));
 	cursorPtr->info.cursor = (Tk_Cursor) cursor;
 	cursorPtr->display = display;
     }
